#include <iostream>
#include <fstream>
#include <math.h>
using namespace std;

#include "accessories.h"


int main()
{
	//=========================================================================================================================
	//
	int i, j, k, t;
	
	// Density
	GridData rho;
	GridData rho_x_x; // 2nd derivatives
	GridData rho_y_y;
	
	// Pressure
	GridData P;
	GridData P_x; // 1st Derivative
	GridData P_y; 
	
	//Velocity
	GridData Vx;
	GridData Vy;
	GridData Vx_x; // 1st Derivatives
	GridData Vy_x;
	GridData Vx_y;
	GridData Vy_y;
	GridData Vx_x_x; // 2nd Derivatives
	GridData Vx_y_y;
	GridData Vy_x_x;
	GridData Vy_y_y;
	
	//T tensor
	GridData Txx;
	GridData Txy;
	GridData Tyx;
	GridData Tyy;
	GridData Txx_x;
	GridData Txy_y;
	GridData Tyx_x;
	GridData Tyy_y;
	
	//Force
	GridData Fx;
	GridData Fy;
	
	
	
	
	
	//=========================================================================================================================
	// Initial Condition
	
	for (i=1; i<=NumXPoints; i++) {
		for (j=1; j<=NumYPoints; j++) {
			//double x = (i - NumXPoints/2.0)*dx;
			//double y = (j - NumYPoints/2.0)*dy;
			// Density
			//rho[i][j] = rho_const*exp(-(x*x + y*y)/100);
			rho[i][j] = rho_const;
			//Tensor
			Txx[i][j] = 0;
			Tyy[i][j] = 0;
			Txy[i][j] = 0;
			Txy[i][j] = 0;
			
			Txx_x[i][j] = 0;
			Tyy_y[i][j] = 0;
			Txy_y[i][j] = 0;
			Tyx_x[i][j] = 0;
			
		}
	}
	
	
	
	for (i=1; i<=NumXPoints; i++) {
		for (j=1; j<=NumYPoints; j++) {
			Fx[i][j] = 0;
			Fy[i][j] = 0;
		}
	}
	double F_width_X = .05; // width of the force distribution
	double F_width_Y = .002; // width of the force distribution
	double sx = 0, sy = 0;
	for (i=1; i<=NumXPoints; i++) {
		for (j=1; j<=NumYPoints; j++) {
			
		
			sx = Lx/2.0 - i*dx;
			sy = Ly/2.0 - j*dy;
			Fx[i][j] = fx*sx*exp(-sx*sx/F_width_X)*exp(-sy*sy/F_width_Y);
			
		
			//Fx[i][12] = fx;
		}
	}
	
	for (i=1; i<=NumXPoints; i++) {
		for (j=1; j<=NumYPoints; j++) {
			Vx[i][j] = 0;
			Vy[i][j] = 0;
		}
	}
	
	
	ofstream chkfile;
	chkfile.open("Check_Force.m");
	chkfile.precision(6);
	chkfile<<"F_width_X = "<<F_width_X<<";"<<endl;
	chkfile<<"F_width_Y = "<<F_width_Y<<";"<<endl;
	chkfile<<"Fx = [];"<<endl;
	for(j=1; j<=NumXPoints; j++)
	{
		chkfile<<"Fx=[Fx; ";
		for(k=1; k<=NumYPoints; k++)
		{
			chkfile<<Fx[j][k]<<" ";
		}
		chkfile<<"];"<<endl;
	}
	chkfile<<"figure, surf(Fx), shading interp;"<<endl;
	chkfile.close();
	
	//return 0;

	
	
	
	//=========================================================================================================================
	
	
	//=========================================================================================================================
	ofstream* file = new ofstream("Dataout.m");

	(*file)<<"dt = "<<dt<<";"<<endl;
	(*file)<<"dx = "<<dx<<";"<<endl;
	(*file)<<"dy = "<<dy<<";"<<endl;
	(*file)<<"NumPointsX = "<<NumXPoints<<";"<<endl;
	(*file)<<"NumPointsY = "<<NumYPoints<<";"<<endl;
	
	(*file)<<"Lx = "<<Lx<<";"<<endl;
	(*file)<<"Ly = "<<Ly<<";"<<endl;
	
	(*file)<<"Re = "<<Re<<";"<<endl;
	(*file)<<"We = "<<We<<";"<<endl;
	(*file)<<"beta = "<<beta<<";"<<endl;
	(*file)<<"fx = "<<fx<<";"<<endl;
	
	
	(*file)<<"Vx_mat = zeros("<<NumYPoints<<", "<<NumFrames<<");"<<endl;
	(*file)<<"V_cent = [];"<<endl;
	(*file)<<"Vx_array = zeros("<<NumXPoints<<", "<<NumYPoints<<", "<<NumFrames<<");"<<endl;
	(*file)<<"Vy_array = zeros("<<NumXPoints<<", "<<NumYPoints<<", "<<NumFrames<<");"<<endl;
	
	
	(*file)<<"Time_Array = [];"<<endl;

	
	file->close();
	delete file;
	
	//=========================================================================================================================

	
	// Time Loop
	for (t=0; t<=Num_Iterations; t++) {
		
		if( !(t/100.0-t/100) ) { cout<<"Iteration # "<<t<<" Time = "<<t*dt<<endl;
		}
		
		for (i=1; i<=NumXPoints; i++) {
			for (j=1; j<=NumYPoints; j++) {
				Vx_x[i][j] = 0;
				Vy_y[i][j] = 0;
				Vx_y[i][j] = 0;
				Vx_y[i][j] = 0;
				
				Vx_x_x[i][j] = 0;
				Vx_y_y[i][j] = 0;
				Vy_x_x[i][j] = 0;
				Vy_y_y[i][j] = 0;
				
				P[i][j] = 0;
				rho_x_x[i][j] = 0;
				rho_y_y[i][j] = 0;
				
			}
		}
		
		
		
		Update_V(&Vx);
		Update_V(&Vy);
		Update_S(&rho);
		//Calculate the derivatives of V
		D_Dx(&Vx, &Vx_x);
		D_Dx(&Vy, &Vy_x);
		D_Dy(&Vx, &Vx_y);
		D_Dy(&Vy, &Vy_y);
		
		// Calculate 2nd derivatives of V
		DD_DDx(&Vx, &Vx_x_x);
		DD_DDy(&Vx, &Vx_y_y);
		DD_DDx(&Vy, &Vy_x_x);
		DD_DDy(&Vy, &Vy_y_y);
		
		
		// Calculate derivatives of rho
		DD_DDx(&rho, &rho_x_x);
		DD_DDy(&rho, &rho_y_y);
		
		
		double tmp_a = 0, tmp_b = 0, tmp_c = 0;
		// Update Elastic Tensor
		for (i=1; i<=NumXPoints; i++) {
			for (j=1; j<=NumYPoints; j++) {
				
				//tmp_a = ((2.0*Vx_x[i][j] - Txx[i][j])/We + 2.0*(Vx_x[i][j]*Txx[i][j] + Vx_y[i][j]*Tyx[i][j]));
				//tmp_b = ((Vy_x[i][j] + Vx_y[i][j] - Txy[i][j])/We + Vx_x[i][j]*Txy[i][j] + Vx_y[i][j]*Tyy[i][j] + Vy_x[i][j]*Txx[i][j] + Vy_y[i][j]*Tyx[i][j]);
				//tmp_c = ((2.0*Vy_y[i][j] - Tyy[i][j])/We + 2.0*(Vy_x[i][j]*Txy[i][j] + Vy_y[i][j]*Tyy[i][j]));
				
				tmp_a = (2.0*Vx_x[i][j] - Txx[i][j])/We;
				tmp_b = (Vy_x[i][j] + Vx_y[i][j] - Txy[i][j])/We;
				tmp_c = (2.0*Vy_y[i][j] - Tyy[i][j])/We;			  
				
				
				Txx[i][j] += tmp_a*dt;
				Txy[i][j] += tmp_b*dt;
				Tyx[i][j] += Txy[i][j];
				Tyy[i][j] += tmp_c*dt;
		
			}
		}
		
		
		// Update Density
		for (i=1; i<=NumXPoints; i++) {
			for (j=1; j<=NumYPoints; j++) {
				rho[i][j] += dt*D*(rho_x_x[i][j] + rho_y_y[i][j]) - dt*rho[i][j]*(Vx_x[i][j] + Vy_y[i][j]);
			}
		}
		
		// Update Pressure
		for (i=1; i<=NumXPoints; i++) {
			for (j=1; j<=NumYPoints; j++) {
				//P[i][j] = (rho[i][j] - rho_const)*eta;
				//P[i][j] = eta*eta*rho[i][j]*rho[i][j]/(2*rho_const);
				P[i][j] = rho[i][j]*rho[i][j]; // dimentionless
			}
		}
		
		// Calculate derivatives of Pressure
		D_Dx(&P, &P_x);
		D_Dy(&P, &P_y);
		
		// Calculate derivatives of Elastic Tensor
		D_Dx(&Txx, &Txx_x);
		D_Dy(&Txy, &Txy_y);
		D_Dx(&Tyx, &Tyx_x);
		D_Dy(&Tyy, &Tyy_y);
		
		
		//Update the velocity
		for (i=1; i<=NumXPoints; i++) {
			for (j=1; j<=NumYPoints; j++) {
				//Vx[i][j] += (-P_x[i][j] + mu_s*(Vx_x_x[i][j] + Vx_y_y[i][j]) + fx)*1.0/(.001 + rho[i][j])*dt/eps;
				//Vy[i][j] += (-P_y[i][j] + mu_s*(Vy_x_x[i][j] + Vy_y_y[i][j]) + fy)*1.0/(.001 + rho[i][j])*dt/eps;
				//Dimentionless
				Vx[i][j] += -P_x[i][j]/(2*M*M)*1.0/rho[i][j]*dt;
				Vx[i][j] += (Vx_x_x[i][j] + Vx_y_y[i][j])/Re*dt*(1-beta);
				Vx[i][j] += (Txx_x[i][j] + Txy_y[i][j])/Re*dt*beta;
				Vx[i][j] += Fx[i][j]*dt;
				
				Vy[i][j] += -P_y[i][j]/(2*M*M)*1.0/rho[i][j]*dt;
				Vy[i][j] += (Vy_x_x[i][j] + Vy_y_y[i][j])/Re*dt*(1-beta);
				Vy[i][j] += (Tyx_x[i][j] + Tyy_y[i][j])/Re*dt*beta;
				Vy[i][j] += Fy[i][j]*dt;

			}
		}
		
		//=========================================================================================================================
		//=========================================================================================================================
		// Record the Data
		if (fmod(t, Sample_time) == 0) {
			ofstream myfile;
			myfile.open("Dataout.m", ios::app);
			myfile.precision(6);
			
			myfile<<"Time_Array = [Time_Array;"<<t*dt<<"  ];"<<endl;
			
			
			myfile<<"Vx = [];"<<endl;
			for(j=1; j<=NumXPoints; j++)
			{
				myfile<<"Vx=[Vx; ";
				for(k=1; k<=NumYPoints; k++)
				{
					myfile<<Vx[j][k]<<" ";
				}
				myfile<<"];"<<endl;
			}
			/*
			myfile<<"Vy = [];"<<endl;
			for(j=1; j<=NumXPoints; j++)
			{
				myfile<<"Vy=[Vy; ";
				for(k=1; k<=NumYPoints; k++)
				{
					myfile<<Vy[j][k]<<" ";
				}
				myfile<<"];"<<endl;
			} */
			
			
			/*myfile<<"Vx = [";
			for (j=1; j<=NumYPoints; j++) {
				myfile<<Vx[30][j]<<" ";
			}
			myfile<<"];"<<endl;
			
			myfile<<"V_cent = [V_cent;"<<Vx[30][50]<<"];"<<endl;*/
			 
			
						
			//myfile<<"Vx_mat(:, "<<Frame_Count<<") = Vx;"<<endl;
			myfile<<"Vx_array(:, :, "<<Frame_Count<<") = Vx;"<<endl;
			//myfile<<"Vy_array(:, :, "<<Frame_Count<<") = Vy;"<<endl;
			myfile<<"end_index = "<<Frame_Count<<";"<<endl;
			Frame_Count++;
			myfile.close();
		}
	}

	return 0;
}


