#include "nrutil.h"
#include "nrutil_.h"
const double Pi = 3.1415926535897932385;



//======================================================================
//Parameters
const double Lx = 3.0; // Periodic BC
const double Ly = 1.0; // Normal BC

const int NumXPoints = 300;
const int NumYPoints = 100;

double dx = Lx/(NumXPoints);
double dy = Ly/(NumYPoints);

double dt = .000001;

const int Num_Iterations = 200000;

// Note: for current discritization its about .73MB per frame
const int Sample_time = 10000;

const double D = 0;//.5; //Diffusion
const double eta = 1.0; // Presure/Density relationship
const double eps = 1.0; // small parameter for velocity
const double lambda = 0; // elastic relaxation time
const double mu_m = 0; // elastic constant
const double mu_s = 20.0; // viscous constant

const double epsilon = .001; // small parameter for stability

const double rho_const = 1.0;

double F_width = .05; // width of the force distribution
const double d = .1; // where the cut is 
const double fx = .1; // force in the x direction
const double fy = 0; //force in the y direction

const double M = 0.1;
const double Re = .05;
const double beta = .6; // retardation ratio
const double We = 1; // Weissenberg Number

const int NumFrames = 200;
int Frame_Count = 1;
//======================================================================

class GridData
{
public:
	GridData();
	
	double * operator[] (int i) {return f[i];}
	
	void operator *= (double f);
	
	void operator += (GridData * GD);
	
	void operator = (GridData * GD);
	
	double ** get_f() {return f;};
	
private:
	double ** f; 
};

void GridData::operator += (GridData * GD)
{
	int i=0, j=0;
	
	for(i=1; i<=NumXPoints; i++)
	{
		for(j=1; j<=NumYPoints; j++)
		{
			f[i][j] += (*GD)[i][j];
		}
	}
	
}

GridData::GridData()
{
	f  = matrix(1,NumXPoints,1,NumYPoints);
}

void GridData::operator *=(double fl)
{
	int i=0, j=0;
	
	for(i=1; i<=NumXPoints; i++)
	{
		for(j=1; j<=NumYPoints; j++)
		{
			f[i][j] *= fl;
		}
	}

}

void GridData::operator =(GridData * GD)
{
	int i=0, j=0;
	
	for(i=1; i<=NumXPoints; i++)
	{
		for(j=1; j<=NumYPoints; j++)
		{
			f[i][j] = (*GD)[i][j];
		}
	}

}


// Periodic Derivatives for X
void D_Dx(GridData* GD, GridData* Result)
{
	
	int Nxt, Prv;
	int i=0, j=0;
	for (i=1; i<=NumXPoints; i++) {
		for (j=2; j<NumYPoints; j++) {
			
			Prv = i-1;
			if (Prv < 1) Prv += NumXPoints;
			Nxt = i+1;
			if (Nxt > NumXPoints) Nxt -= NumXPoints;
			
			(*Result)[i][j] = ((*GD)[Nxt][j] - (*GD)[Prv][j])/(2.0*dx);
			
		}
	}

}

void DD_DDx(GridData* GD, GridData* Result)
{
	int Nxt, Prv;
	int i=0, j=0;
	for (i=1; i<=NumXPoints; i++) {
		for (j=2; j<NumYPoints; j++) {
			
			Prv = i-1;
			if (Prv < 1) Prv += NumXPoints;
			Nxt = i+1;
			if (Nxt > NumXPoints) Nxt -= NumXPoints;
			
			(*Result)[i][j] = ((*GD)[Nxt][j] + (*GD)[Prv][j] - 2.0 * (*GD)[i][j])/(dx*dx);
			//(*Result)[i][j] = ((*GD)[Nxt][j] + (*GD)[Prv][j] - 2.0 * (*GD)[i][j])/(4.0*dx*dx);
		}
	}
}

// Reflective derivatives for Y
void D_Dy(GridData* GD, GridData* Result)
{

	int i=0, j=0;
	for (i=1; i<=NumXPoints; i++) {
		for (j=2; j<NumYPoints; j++) {
			
			(*Result)[i][j] = ((*GD)[i][j+1] - (*GD)[i][j-1])/(2.0*dy);
			
		}
	}
}

void DD_DDy(GridData* GD, GridData* Result)
{
	
	int i=0, j=0;
	for (i=1; i<=NumXPoints; i++) {
		for (j=2; j<NumYPoints; j++) {
			
			(*Result)[i][j] = ((*GD)[i][j+1] + (*GD)[i][j-1] - 2.0 * (*GD)[i][j])/(dy*dy);
			//(*Result)[i][j] = ((*GD)[i][j+1] + (*GD)[i][j-1] - 2.0 * (*GD)[i][j])/(4.0*dy*dy);
		}
	}
}


void Update_V(GridData* V)
{
	int i=0;
	for (i=1; i<=NumXPoints; i++) {
		(*V)[i][1] = -1.0 * (*V)[i][3];
		(*V)[i][2] = 0.0;
		(*V)[i][NumYPoints-1] = 0.0;
		(*V)[i][NumYPoints] = -1.0 * (*V)[i][NumYPoints-2];
	}
}

void Update_S(GridData* S)
{
	int i=0;
	for (i=1; i<=NumXPoints; i++) {
		(*S)[i][1] = (*S)[i][2];
		(*S)[i][NumYPoints] = (*S)[i][NumYPoints-1];
	}
}






